@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget-xine.c,v 1.205 2005/09/02 11:22:01 hadess Exp $
+ * $Id: bacon-video-widget-xine.c,v 1.210.2.2 2006/03/09 15:27:26 hadess Exp $
  *
  * the xine engine in a widget - implementation
  */
@@ -34,6 +34,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <sched.h>
 /* X11 */
 #include <X11/X.h>
 #include <X11/Xlib.h>
@@ -1420,6 +1421,43 @@ xine_event (void *user_data, const xine_event_t *event)
 	}
 }
 
+static int
+bacon_video_widget_sort_queue (gconstpointer a, gconstpointer b, gpointer data)
+{
+	signal_data *one, *two;
+
+	one = (signal_data *) a;
+	two = (signal_data *) b;
+
+	if (one->signal == two->signal)
+		return 0;
+	if (one->signal == ERROR_ASYNC || one->signal == MESSAGE_ASYNC)
+		return 1;
+	return -1;
+}
+
+#if (!(GLIB_CHECK_VERSION(2,9,1)))
+static void
+bacon_video_widget_queue_sort (GAsyncQueue *queue)
+{
+	GList *list = NULL, *l;
+	signal_data *data;
+
+	while ((data = g_async_queue_try_pop (queue)) != NULL) {
+		list = g_list_insert_sorted (list, data, (GCompareFunc) bacon_video_widget_sort_queue);
+	}
+
+	if (list == NULL)
+		return;
+
+	for (l = list; l != NULL; l = l->next) {
+		g_async_queue_push (queue, l->data);
+	}
+	g_list_free (list);
+}
+
+#endif /* ! GLIB_CHECK_VERSION 2.9.1 */
+
 static void
 xine_try_error (BaconVideoWidget *bvw, gboolean probe_error, GError **error)
 {
@@ -1428,12 +1466,21 @@ xine_try_error (BaconVideoWidget *bvw, gboolean probe_error, GError **error)
 
 	save_data = NULL;
 
+	sched_yield ();
+
+	/* Sort the queue with the errors first */
+#if GLIB_CHECK_VERSION(2,9,1)
+	g_async_queue_sort (bvw->priv->queue, bacon_video_widget_sort_queue, bvw);
+#else
+	bacon_video_widget_queue_sort (bvw->priv->queue);
+#endif
+
 	/* Steal messages from the async queue, if there's an error,
 	 * to use as the error message rather than the crappy errors from
 	 * xine_open() */
 	while ((data = g_async_queue_try_pop (bvw->priv->queue)) != NULL)
 	{
-		if (data->signal == ERROR_ASYNC)
+		if (data->signal == ERROR_ASYNC || data->signal == MESSAGE_ASYNC)
 		{
 			if (save_data != NULL)
 			{
@@ -1442,8 +1489,8 @@ xine_try_error (BaconVideoWidget *bvw, gboolean probe_error, GError **error)
 			}
 			save_data = data;
 		} else {
-			g_free (data->msg);
-			g_free (data);
+			g_async_queue_push (bvw->priv->queue, data);
+			break;
 		}
 	}
 
@@ -1467,6 +1514,9 @@ xine_try_error (BaconVideoWidget *bvw, gboolean probe_error, GError **error)
 	switch (err)
 	{
 	case XINE_ERROR_NO_INPUT_PLUGIN:
+		g_set_error (error, BVW_ERROR, BVW_ERROR_NO_PLUGIN_FOR_FILE,
+				_("There is no input plugin to handle the location of this movie"));
+		break;
 	case XINE_ERROR_NO_DEMUX_PLUGIN:
 		g_set_error (error, BVW_ERROR, BVW_ERROR_NO_PLUGIN_FOR_FILE,
 				_("There is no plugin to handle this movie."));
@@ -1962,11 +2012,9 @@ bacon_video_widget_get_nice_codec_name (BaconVideoWidget *bvw,
 
 char *
 bacon_video_widget_get_backend_name (BaconVideoWidget *bvw)
-{	
-	return g_strdup_printf ("xine-lib version %d.%d.%d",
-			XINE_MAJOR_VERSION,
-			XINE_MINOR_VERSION,
-			XINE_SUB_VERSION);
+{
+	return g_strdup_printf ("xine-lib version %s",
+			xine_get_version_string ());
 }
 
 static char *
@@ -1989,7 +2037,14 @@ bacon_video_widget_open_with_subtitle (BaconVideoWidget *bvw, const char *mrl,
 	g_return_val_if_fail (bvw->priv->mrl == NULL, FALSE);
 
 	bvw->priv->got_redirect = FALSE;
-	bvw->priv->mrl = g_strdup (mrl);
+
+	/* Hack to get VCD playback from .cue files */
+	if (g_str_has_prefix (mrl, "vcd:/") != FALSE
+			&& g_str_has_suffix (mrl, ".cue") != FALSE) {
+		bvw->priv->mrl = g_strdup_printf ("%s@", mrl);
+	} else {
+		bvw->priv->mrl = g_strdup (mrl);
+	}
 
 	if (subtitle_uri != NULL)
 	{
@@ -2192,7 +2247,7 @@ gboolean bacon_video_widget_seek_time (BaconVideoWidget *bvw, gint64 time,
 		return TRUE;
 	}
 
-	if (time > length) {
+	if (time > length && g_str_has_prefix (bvw->priv->mrl, "dvd:") == FALSE && g_str_has_prefix (bvw->priv->mrl, "vcd:") == FALSE) {
 		signal_data *data;
 
 		data = g_new0 (signal_data, 1);
@@ -2537,7 +2592,6 @@ bacon_video_widget_set_show_cursor (BaconVideoWidget *bvw,
 {
 	g_return_if_fail (bvw != NULL);
 	g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
-	g_return_if_fail (bvw->priv->xine != NULL);
 
 	if (show_cursor == FALSE)
 	{
@@ -3548,7 +3602,7 @@ GList
 	num_channels = xine_get_stream_info
 		(bvw->priv->stream, XINE_STREAM_INFO_MAX_SPU_CHANNEL);
 
-	if (num_channels < 2)
+	if (num_channels < 1)
 		return NULL;
 
 	for(i = 0; i < num_channels; i++)
